<div class="row">
  <div class="col-xs-12 col-md-offset-3 col-md-6">
    <h2 class="text-center">
      <i class="fa fa-gift" title="Package"></i>
      {{ name }}
    </h2>
  </div>
</div>
<div class="row">
  <div class="col-xs-12 col-md-6" ng-if="userTableArgs">
    <div ng-include="PARTIAL + 'pretty_table.html'" ng-init="tableArgs = userTableArgs"></div>
  </div>

  <div class="col-xs-12 col-md-6" ng-if="groupTableArgs">
    <div ng-include="PARTIAL + 'pretty_table.html'" ng-init="tableArgs = groupTableArgs"></div>
    <div ng-show="groupTableArgs.items.length === 0">
      <p ng-repeat="group in DEFAULT_READ">
        default group '{{ group }}' has read access
      </p>
    </div>
  </div>
</div>
